--[[
    Mount by LordFarlander

    Based on Kennel by Tekkub Stoutwrithe.
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = AceLibrary( "AceLocale-2.2" ):new( "Stable" );

local options = {
    type = "group",
    args = {
        [L["toggleSwapMiniPets"]] = {
            type = "toggle",
            name = L["Swap Mini Pets"],
            desc = L["Enable/disable the swapping of mini pets."],
            get = function()
                      return Stable.db.char.DoSwap["MiniPets"];
                  end,
            set = function( val )
                      Stable.db.char.DoSwap["MiniPets"] = not Stable.db.char.DoSwap["MiniPets"];
                  end,
        },
        [L["toggleSwapGroundMounts"]] = {
            type = "toggle",
            name = L["Swap Ground Mounts"],
            desc = L["Enable/disable the swapping of ground mounts."],
            get = function()
                      return Stable.db.char.DoSwap["GroundMounts"];
                  end,
            set = function( val )
                      Stable.db.char.DoSwap["GroundMounts"] = not Stable.db.char.DoSwap["GroundMounts"];
                  end,
        },
        [L["toggleSwapFlyingMounts"]] = {
            type = "toggle",
            name = L["Swap Flying Mounts"],
            desc = L["Enable/disable the swapping of flying mounts."],
            get = function()
                      return Stable.db.char.DoSwap["FlyingMounts"];
                  end,
            set = function( val )
                      Stable.db.char.DoSwap["FlyingMounts"] = not Stable.db.char.DoSwap["FlyingMounts"];
                  end,
        },
    },
};

Stable = AceLibrary( "AceAddon-2.0" ):new( "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0" );
Stable:RegisterChatCommand( "/Stable", "/stbl", options );
Stable:RegisterDB( "StableDB", "StableDBPC" );
Stable:RegisterDefaults( "char", {
    DoSwap = { ["MiniPets"] = true, ["GroundMounts"] = true, ["FlyingMounts"] = true, },
    } );

local PetAndMountDatabase = AceLibrary( "PetAndMountDatabase-1.0" );

Stable.bankopen = false;
Stable.swapped = false;
Stable.sets = {
    ["MiniPets"] = {
        normal = { player = {bag = {}, slot = {}, num = 0 }, bank = {bag = {}, slot = {}, num = 0 }, set = "MiniPets.Normal", },
        holiday = { player = {bag = {}, slot = {}, num = 0 }, bank = {bag = {}, slot = {}, num = 0 }, set = "MiniPets.Holiday", },
    },
    ["GroundMounts"] = {
        slow = { player = { bag = {}, slot = {}, num = 0 }, bank = { bag = {}, slot = {}, num = 0 }, set = "Mounts.Ground.Slow", },
        fast = { player = { bag = {}, slot = {}, num = 0 }, bank = { bag = {}, slot = {}, num = 0 }, set = "Mounts.Ground.Fast", },
        veryfast = { player = { bag = {}, slot = {}, num = 0 }, bank = { bag = {}, slot = {}, num = 0 }, set = "Mounts.Ground.VeryFast", },
    },
    ["FlyingMounts"] = {
        fast = { player = { bag = {}, slot = {}, num = 0 }, bank = { bag = {}, slot = {}, num = 0 }, set = "Mounts.Flying.Fast", },
        veryfast = { player = { bag = {}, slot = {}, num = 0 }, bank = { bag = {}, slot = {}, num = 0 }, set = "Mounts.Flying.VeryFast", },
        extremelyfast = { player = { bag = {}, slot = {}, num = 0 }, bank = { bag = {}, slot = {}, num = 0 }, set = "Mounts.Flying.ExtremelyFast", },
    },
};

function Stable:OnEnable()
    self:RegisterEvent( "BANKFRAME_OPENED",
        function()
            local stableSets = Stable.sets;
            local doSwap = Stable.db.char.DoSwap;

            Stable.bankopen = true;
            Stable.swapped = false;

            for _, set in pairs( stableSets ) do
                for _, subset in pairs( set ) do
                    subset.player.num = 0;
                    subset.bank.num = 0;
                end--for
            end--for
            for bag = BANK_CONTAINER, NUM_BAG_SLOTS + NUM_BANKBAGSLOTS do
                local bagset = bag <= NUM_BAG_SLOTS and bag >= 0 and "player" or "bank";

                for slot = 1, GetContainerNumSlots( bag ) do
                    for setName, set in pairs( stableSets ) do
                        if( doSwap[setName] ) then
                            for _, subset in pairs( set ) do
                                if( PetAndMountDatabase:ItemInSet( GetContainerItemLink( bag, slot ), subset.set ) ) then
                                    local petSet = subset[bagset];
                                    local n = petSet.num + 1;

                                    petSet.num = n;
                                    petSet.bag[n] = bag;
                                    petSet.slot[n] = slot;
                                end
                            end--for
                        end--if
                    end--for
                end--for
            end--for
            for setName, set in pairs( stableSets ) do
                if( doSwap[setName] ) then
                    for _, subset in pairs( set ) do
                        Stable.swapped = Stable:DoSwaps( subset ) or Stable.swapped;
                    end--for
                end--if
            end--for
        end );
    self:RegisterEvent( "BANKFRAME_CLOSED",
        function()
            if( Stable.bankopen and Stable.swapped and FuBar_CorkFu ) then
                FuBar_CorkFu:GetModule( "Minipet" ):ActivatePet();
            end--if
            Stable.bankopen = false;
        end );
end

------------------------------
--     Helper Functions     --
------------------------------
function Stable:DoSwaps( set )
    if set.player.num > set.bank.num then
        set.player, set.bank = set.bank, set.player;
    end

    local p, b = set.player, set.bank;

    if( p.num == 0 ) then
        return false;
    end
    for i = 1, p.num do
        local r = math.random( b.num + 1 - i );

        PickupContainerItem( p.bag[i], p.slot[i] );
        PickupContainerItem( b.bag[r], b.slot[r] );
        table.remove( b.bag, r );
        table.remove( b.slot, r );
    end
    
    return ( p.num > 0 ) and ( b.num > 0 );
end